/************************************************************************
*
* Module:       	BMP085_2.C
* Description:  	Code to determine BMP085 functionality.
* Line length:  	120 characters [only if the length is longer than 80 chars]
* Functions:    	See Below
*
* Date:				Authors:					Comments:
* 21 Aug 2011   	Austin Schaller    			Created
*					Nick ODell
*
************************************************************************/
#define 	_LEGACY_HEADERS 
#include	<htc.h>
#include 	<pic16f690.h>
#include	<math.h>
#include	<stdlib.h>
#include 	<stdio.h>
#include 	<string.h>
#include 	"i2c.h"
#include 	<math.h>		// Required for altitude measurement
#include 	"BMP085_Functions.h"

__CONFIG (MCLRDIS & UNPROTECT & WDTDIS & FOSC_HS);

//** Defines ************************************************************
#define 	BMP085_R 	0xEF
#define 	BMP085_W 	0xEE
#define 	OSS 		0		// Oversampling Setting (note: code is not set up to use other OSS values)

#define TRIS_DATA TRISB4    //data port condition: input/output
#define TRIS_SCK  TRISB6
#define RX_PIN TRISB5
#define TX_PIN TRISB7 
#define DATA RB4         //SHT11 DATA pin
#define SCK RB6          //STH11 SCK pin 

#define XTAL_FREQ  11059200
#define noACK 0
#define ACK 1
//** Globals Variables **************************************************
long bmp085ReadShort(unsigned char address);
void BMP085_Calibration(void);
long bmp085ReadTemp(void);
long bmp085ReadPressure(void);
void bmp085Convert(long *temperature, long *pressure);
// Voodoo calibration varibles
short ac1;
short ac2;
short ac3;
short b1;
short b2;
short mb;
short mc;
short md;
unsigned short ac4;
unsigned short ac5;
unsigned short ac6;
int i,huy;
unsigned char buffer[10]={0};
unsigned char buffer1[10]={0};



typedef union 
{ unsigned int i;
  float f;
} value;
 struct
{
	float Float;
	int Int;
	int Dec;
}Number;

void Init_port(void)
{
    TRISA = 0x00;
    TRISB = 0x00;
    TRISC = 0x00;
 	INTCON  = 0XA0;
    ANSEL=0x00; //Digital AN[0:7]
    ANSELH = 0x00; //Digital AN[13:8]
    ADCON0=0; //turn off A/D
}
void init_USART(void)
{

	RX_PIN = 1;	
	TX_PIN = 1;	
//	BRG16 =1;	 
	SPBRG =17;     //if osc freq = 8Mhz{103 = 1200bps;  51 = 2400bps;	12 = 9600bps}
	RCSTA = 0x90;	//if osc freq = 11.0592 MHz{9600bps=17;}
	TXSTA = 0x20;
}

void putch(unsigned char byte) 
{
	/* output one byte */
	while(!TXIF)	/* set when register is empty */
		continue;
	TXREG = byte;
}

unsigned char getch() {
	/* retrieve one byte */
	while(!RCIF)	/* set when register is not empty */
		continue;
	return RCREG;	
}
 void DelayUs(unsigned char x)	
{
 	unsigned int d = 0;
	d = (x/((11059200)/(XTAL_FREQ))|1);
	while(--d != 0);
	
}
void DelayMs(unsigned char cnt)
{
	unsigned char	i;
	do {
		i = 4;
		do {
				DelayUs(250);
			} while(--i);
		} while(--cnt);

}
void Transmit_data(float data)
{
					
					
				
				//	Number.Float = data; 
				//Number.Int = Number.Float;
					itoa(buffer,data,10);
					//huy=strlen(buffer);
//					for(i=0;i<7;i++)
//					{
//							if(i==huy-2)
//							{	
//								buffer1[i+1]=buffer[i];
//								buffer1[i]='.';
//								
//							}
//							 if (i>huy-2)
//							{	buffer1[i+1]=buffer[i];
//							}
//							if(i<huy-2)
//							{
//								buffer1[i]=buffer[i];
//							}
//					}
					putch(buffer[0]);
					putch(buffer[1]);
					putch(buffer[2]);
					putch(buffer[3]);
					putch(buffer[4]);
					putch(buffer[5]);
					putch(buffer[6]);
					putch(buffer[7]);
					
	
}
unsigned char s_write_byte(unsigned char value)
//------------------------------------------------------------------------------	
 
{ 
   unsigned char i,error=0;
   TRIS_DATA = 0;   //SHT11 WRITE...make DATA-line output
   SCK=0; //initial state
   for (i=0x80;i>0;i/=2) //shift bit for masking
   {
     DelayUs(5);
     if (i & value)
         DATA=1; //masking value with i , write to SENSI-BUS
     else
         DATA=0;
     SCK=1; //clk for SENSI-BUS
 
     DelayUs(5);   //pulswith approx. 5 us
 
     SCK=0;
   }
 
 
   TRIS_DATA = 1;    //SHT11 READ..make DATA-line input
   DelayUs(5);
   SCK=1; //clk #9 for ack
   DelayUs(5);
   error = DATA; //check ack (DATA will be pulled down by SHT11)
   SCK=0;
   return error; //error=1 in case of no acknowledge
}
 
 
/****************************************************************************/
// reads a byte from the Sensibus and gives an acknowledge in case of "ack=1"
/****************************************************************************/
 
unsigned char s_read_byte(unsigned char ack)
{
   unsigned char i,val=0;
   TRIS_DATA = 1;        //SHT11 READ
   //DATA=1; //release DATA-line
   for (i=0x80;i>0;i/=2) //shift bit for masking
   {
     DelayUs(5);
     SCK=1; //clk for SENSI-BUS
     if (DATA)
     val=(val | i); //read bit		
     DelayUs(5);
     SCK=0;
   }
   TRIS_DATA = 0;    //make DATA-Line output
   DATA=!ack;         //in case of "ack==1" pull down DATA-Line
   SCK=1; //clk #9 for ack 
   DelayUs(5); //pulswith approx. 5 us
   SCK=0;
   TRIS_DATA=1;
   //DATA=1; //release DATA-line
   return val;
}

//----------------------------------------------------------------------------------
void s_transstart(void)
//----------------------------------------------------------------------------------
// generates a transmission start
//       _____         ________
// DATA:      |_______|
//           ___     ___
// SCK : ___|   |___|   |______
{
   TRIS_DATA = 0;   //SHT11 WRITE - DATA_Line output
 
   DATA=1;
   SCK=0;   //Initial state
   DelayUs(5);
   SCK=1;
   DelayUs(5);
   DATA=0;
   DelayUs(5);
   SCK=0;
   DelayUs(5);
   SCK=1;
   DelayUs(5);
   DATA=1;
   DelayUs(5);
   SCK=0;
} 
//void i2c_Stop(void)
//{
//	/* don't assume SCL is high on entry */
//
//	SCL_LOW();
//	SDA_LOW();					/* ensure data is low first */
//	
//	__delay_us(I2C_TM_DATA_SU);
//	SCL_DIR = I2C_INPUT;		/* float clock high */
//	__delay_us(I2C_TM_STOP_SU);
//	SDA_HIGH();					/* the low->high data transistion */
//	__delay_us(I2C_TM_BUS_FREE);	/* bus free time before next start */
//	SDA_DIR = I2C_INPUT;		/* float data high */
//
//	return;
//}
//unsigned char i2c_Open(unsigned char address, unsigned char mode)
//{
//	i2c_Start();
//	i2c_SendAddress(address, mode);
//	if(i2c_ReadAcknowledge()) 
//		return TRUE;
//
//	return FALSE;
//}

//----------------------------------------------------------------------------------
void s_connectionreset(void)
//----------------------------------------------------------------------------------
// communication reset: DATA-line=1 and at least 9 SCK cycles followed by transstart
//       _____________________________________________________         ________
// DATA:                                                      |_______|
//          _    _    _    _    _    _    _    _    _        ___     ___
// SCK : __| |__| |__| |__| |__| |__| |__| |__| |__| |______|   |___|   |______
{
   unsigned char i;
   TRIS_DATA = 0;  //SHT11 WRITE
   DATA=1; SCK=0;  //Initial state
   for(i=0;i<12;i++) //12 SCK cycles....more than 9 for safe being
   {
	 SCK=1;
     DelayUs(5);
     SCK=0;
   }
   s_transstart(); //transmission start
}


// BMP stuff can be included now.
//#include "EOSS_BMP085.c"		// IMPORTANT - Must include functions for BMP085 sensor!

/************************************************************************
*
* Purpose:      Configures USART module for TX operation
* Passed:       SPBRG, TXSTA, RCSTA
* Returned:     None
* Note:			Asynchronous Mode
*
* Date:			Author:				Comments:
* 20 Sep 2011	Austin Schaller     Created
*
************************************************************************/
//void openTxUsart(void)
//{
//	// TX UART Configuration
//	
//	SPBRG = 9;				// Set baud = 9600
//	TXSTAbits.SYNC = 0;		// Asynchronous mode
//	RCSTAbits.SPEN = 1;		// Serial port enabled
//	TXSTAbits.BRGH = 0;		// Low Speed
//	TXSTAbits.TXEN = 1;		// Enable transmission 
//}


//void delay_ms(unsigned short ms)
//{
//	for(; ms > 0; ms--)
//	{
//		Delay100TCYx(20);
//	}
//}

 


/************************************************************************
*
* Purpose:      Calibrates the BMP085 pressure sensor.
* Passed:       Void
* Returned:     Void
* Note:
*
* Date:				Author:				Comments:
* 16 Mar 2011		Austin Schaller     Created
*
************************************************************************/
void BMP085_Calibration(void)
{
	ac1 = bmp085ReadShort(0xAA);
	ac2 = bmp085ReadShort(0xAC);
	ac3 = bmp085ReadShort(0xAE);
	ac4 = bmp085ReadShort(0xB0);
	ac5 = bmp085ReadShort(0xB2);
	ac6 = bmp085ReadShort(0xB4);
	b1 = bmp085ReadShort(0xB6);
	b2 = bmp085ReadShort(0xB8);
	mb = bmp085ReadShort(0xBA);
	mc = bmp085ReadShort(0xBC);
	md = bmp085ReadShort(0xBE);

	/*printf("\tAC1 = %d\r\n", ac1);
	printf("\tAC2 = %d\r\n", ac2);
	printf("\tAC3 = %d\r\n", ac3);
	printf("\tAC4 = %d\r\n", ac4);
	printf("\tAC5 = %d\r\n", ac5);
	printf("\tAC6 = %d\r\n", ac6);
	printf("\tB1 = %d\r\n", b1);
	printf("\tB2 = %d\r\n", b2);
	printf("\tMB = %d\r\n", mb);
	printf("\tMC = %d\r\n", mc);
	printf("\tMD = %d\r\n", md);
	printf("------------------------\r\n");*/
}

/************************************************************************
*
* Purpose:      Will read two sequential 8-bit registers, and return
				a 16-bit value.
* Passed:       Unsigned char
* Returned:     Short
* Note:
*
* Date:				Author:				Comments:
* 16 Mar 2011		Austin Schaller     Created
*
************************************************************************/
long bmp085ReadShort(unsigned char address)
{
	unsigned short msb, lsb;
	unsigned short data;
	
//	StartI2C();
	s_transstart();
	
//	WriteI2C(BMP085_W);		// Write 0xEE
//	WriteI2C(address);		// Write register address
//	i2c_SendByte(BMP085_W);
//	i2c_SendByte(address);

	s_write_byte(BMP085_W);
	s_write_byte(address);
//	RestartI2C();
	s_connectionreset();
	
//	WriteI2C(BMP085_R);		// Write 0xEF
	s_write_byte(BMP085_R);
//	msb = ReadI2C();		// Get MSB result
	msb = s_read_byte(ACK);
//	putch('k');
//	Transmit_data(msb);
	//printf("msb: %d\r\n", msb);
	//AckI2C();
	//lsb = ReadI2C();		// Get LSB result
	lsb = s_read_byte(ACK);
//	putch('a');
//	Transmit_data(lsb);
	//printf("lsb: %d\r\n", lsb);

//	NotAckI2C();
	s_read_byte(noACK);
//	StopI2C();
	//i2c_Stop();
	
	/*data = msb * 256;
	data |= lsb;*/
	data = msb;
	data *= 256;
	data |= lsb;

	//printf("data: %d\r\n", data);
	return data;
}

/************************************************************************
*
* Purpose:      Will read the 16-bit temperature value of BMP085 sensor.
* Passed:       Void
* Returned:     Long
* Note:
*
* Date:				Author:				Comments:
* 16 Mar 2011		Austin Schaller     Created
*
************************************************************************/
long bmp085ReadTemp(void)
{
//	StartI2C();
	s_transstart();	
//	WriteI2C(BMP085_W);		// Write 0xEE
//	WriteI2C(0xF4);		// Write register address
//	WriteI2C(0x2E);		// Write register data for temp
	s_write_byte(BMP085_W);
	s_write_byte(0XF4);	// Write register address
	s_write_byte(0X2E);	// Write register data for temp
	
	//StopI2C();
//	i2c_Stop();
	DelayMs(10);		// Max time is 4.5ms
	
	return bmp085ReadShort(0xF6);
}

/************************************************************************
*
* Purpose:      Will read the 16-bit pressure value from BMP085 sensor.
* Passed:       Void
* Returned:     Long
* Note:
*
* Date:				Author:				Comments:
* 16 Mar 2011		Austin Schaller     Created
*
************************************************************************/
long bmp085ReadPressure(void)
{
//	StartI2C();
	s_transstart();	
//	WriteI2C(BMP085_W);		// Write 0xEE
//	WriteI2C(0xF4);		// Write register address
//	WriteI2C(0x34);		// Write register data for temp
	s_write_byte(BMP085_W);
	s_write_byte(0XF4);	// Write register address
	s_write_byte(0X34);	// Write register data for temp
	
	
//	StopI2C();
//	i2c_Stop();
	DelayMs(5);		// Max time is 4.5ms
	
	return bmp085ReadShort(0xF6);
}

/************************************************************************
*
* Purpose:      Will read the 16-bit pressure value from BMP085 sensor.
* Passed:       Long *temperature, long *pressure
* Returned:     Void
* Note:
*
* Date:				Author:				Comments:
* 16 Mar 2011		Austin Schaller     Created
*
************************************************************************/
void bmp085Convert(long *temperature, long *pressure)
{
	long ut;
	long up;
	long x1, x2, b5, b6, x3, b3, p;
	unsigned long b4, b7;
	
	// Temperature before pressure
	ut = bmp085ReadTemp();
	up = bmp085ReadPressure();
	
	x1 = ((long) ut - ac6) * ac5 >> 15;
	x2 = ((long) mc * 1 << 11) / (x1 + md);
	b5 = x1 + x2;
	
	*temperature = (b5 + 8) >> 4;
	
	b6 = b5 - 4000;
	x1 = (b2 * (b6 * b6 >> 12)) >> 11;
	x2 = ac2 * b6 >> 11;
	x3 = x1 + x2;
	b3 = (((long) ac1 * 4 + x3) + 2) * 1 << 2;
	x1 = ac3 * b6 >> 13;
	x2 = (b1 * (b6 * b6 >> 12)) >> 16;
	x3 = ((x1 + x2) + 2) >> 2;
	b4 = (ac4 * (unsigned long) (x3 + 32768)) >> 15;
	b7 = ((unsigned long) up - b3) * (50000 >> OSS);
	p = b7 < 0x80000000 ? (b7 * 2) / b4 : (b7 / b4) * 2;
	x1 = (p >> 8) * (p >> 8);
	x1 = (x1 * 3038) >> 16;
	x2 = (-7357 * p) >> 16;
	
	*pressure = p + ((x1 + x2 + 3791) >> 4);
	
	DelayMs(5);
}




void main()
{
	long temperature = 0;
	long pressure = 0;
	//long altitude = 0;
	//double temp = 0;

////	// I2C
//	TRISBbits.TRISB6 = 1;		// SCL
//	TRISBbits.TRISB4 = 1;		// SDA
////	
//	// UART
//	TRISBbits.TRISB7 = 1;		// TX
//	TRISBbits.TRISB5 = 1;		// RX
//	
//	// Initialize UART
//	openTxUsart();
	Init_port();
	init_USART();
	// Initialize I2C
//	OpenI2C(MASTER, SLEW_OFF);
	i2c_Open(0XFE,MASTER_MODE);
	BMP085_Calibration();
	
	DelayMs(100);
	TRIS_SCK =0;
	
	while(1)
	{
		s_connectionreset();
		bmp085Convert(&temperature, &pressure);
	//	putch(ac1);
		putch('p');
		Transmit_data(pressure);
		putch('c');
		Transmit_data(temperature);	
	
	//	printf("Temperature: %d (in 0.1 deg C)\r\n", temperature);
	//	printf("Pressure: %d Pa\n\r\n", pressure);

		//printf("\r\n\r\n");
		
		// For fun, lets convert to altitude
		/*temp = (double) pressure/101325;
		temp = 1-pow(temp, 0.19029);
		altitude = round(44330*temp);
		printf("Altitude: %ldm\n\n", altitude);*/
		
		DelayMs(1000);
	}
}